using System;
using System.Threading;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;


namespace Annabooks.SHTC3
{
    public class SHTC3
    {
        private const byte SHTC3_ADDR = 0x70;
        private byte[] TEMPERATURE_CSD = { 0x78, 0x66 };
        private byte[] HUMIDITYMESSURE_CSD = { 0x58, 0xE0 };
        private byte[] WAKESENSOR = { 0x35, 0x17 };
        private byte[] SLEEPSENSOR = { 0xB0, 0x98 };
        private byte[] SOFTRESET = { 0x80, 0x5D };

        private I2cDevice SHTC3Sensor;

        private double tempResult = 0.0;
        private double humidityResult = 0.0;

        private bool SHTC3Ready = false;
        public string connectionError = "";

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public SHTC3(string I2Cport, bool I2CBusShared)
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            I2CInitialize(I2Cport, I2CBusShared);
        }

        private async void I2CInitialize(string I2Cport, bool I2CBusShared)
        {
            try
            {
                I2cConnectionSettings i2cSettings = new I2cConnectionSettings(SHTC3_ADDR);
                i2cSettings.BusSpeed = I2cBusSpeed.StandardMode;
                if (I2CBusShared)//True
                {
                    i2cSettings.SharingMode = I2cSharingMode.Shared;
                }
                else//false
                {
                    i2cSettings.SharingMode = I2cSharingMode.Exclusive; // The device gets exclusive hold for hold master calls
                }
                string aqs = I2cDevice.GetDeviceSelector(I2Cport);
                DeviceInformationCollection i2cDevInfo = await DeviceInformation.FindAllAsync(aqs);
                SHTC3Sensor = await I2cDevice.FromIdAsync(i2cDevInfo[0].Id, i2cSettings);
            }
            catch (Exception ex)
            {
                connectionError = ex.Message;
            }
            if (SHTC3Sensor != null)
            {
                SHTC3Ready = true;
            }
        }

        public double ReadTempC()
        {

            if (SHTC3Ready == false)
            {
                connectionError = "SHTC3 Not ready";
                return 0;
            }

            SHTC3Sensor.Write(WAKESENSOR); //wake the sensor
            Thread.Sleep(10);
            SHTC3Sensor.Write(TEMPERATURE_CSD); //take the messurment
            Thread.Sleep(500);

            byte[] tempBuf = new byte[3]; //get all three bytes, but we will focus on the first two            
            SHTC3Sensor.Read(tempBuf);

            var tempBufFinal = tempBuf[0] << 8;
            tempBufFinal += tempBuf[1];
            tempResult = ((Convert.ToDouble(tempBufFinal) * 175) / Math.Pow(2, 16)) - 45;

            SHTC3Sensor.Write(SLEEPSENSOR); // Put the sensor back to sleep
            return Math.Round(tempResult, 2);

        }

        public double ReadHumidity()
        {
            if (SHTC3Ready == false)
            {
                connectionError = "SHTC3 Not ready";
                return 0;
            }
            SHTC3Sensor.Write(WAKESENSOR); //wake the sensor
            Thread.Sleep(10);
            SHTC3Sensor.Write(HUMIDITYMESSURE_CSD); //take the measurment
            Thread.Sleep(500);

            byte[] humidityBuf = new byte[3];
            SHTC3Sensor.Read(humidityBuf);

            var humidityFinal = humidityBuf[0] << 8;
            humidityFinal += humidityBuf[1];
            humidityResult = ((Convert.ToDouble(humidityFinal) * 100) / Math.Pow(2, 16));

            SHTC3Sensor.Write(SLEEPSENSOR); // Put the sensor back to sleep
            return Math.Round(humidityResult, 2);
        }

        public void HumiditySensorSoftReset()
        {
            SHTC3Sensor.Write(SOFTRESET);
        }

        public double GetLastTemp()
        {
            return Math.Round(tempResult, 2);
        }

        public double GetLastHumidity()
        {
            return Math.Round(humidityResult, 2);
        }

        public bool HTU21DStatus()
        {
            return SHTC3Ready;
        }

    }
}
